@@ -44,7 +44,8 @@ class JythonGenerator:
         if "imread" in step.operation:
             args = step.inputs
         else:
-            args = step.inputs + [f"cle.create_like({step.inputs[0]})"] + step.args
+            inter = "labels_" if step.is_labels else ""
+            args = step.inputs + [f"cle.create_{inter}like({step.inputs[0]})"] + step.args
         return f"{step.output} = cle.{step.operation}({', '.join(map(str, args))})"
 
     @staticmethod
@@ -59,6 +60,81 @@ class JythonGenerator:
     def newline():
         return ""
 
+class NapariPythonGenerator(JythonGenerator):
+    @staticmethod
+    def header():
+        from napari_pyclesperanto_assistant import __version__
+        from textwrap import dedent
+
+        return dedent(
+            f"""
+            # To make this script run in napari's script editor, install 
+            # pyclesperanto_prototype and the script editor
+            #
+            # ```
+            # pip install pyclesperanto_prototype napari-script-editor
+            # ```
+            # Read more: http://clesperanto.net
+            #
+            # Generator (P) version: {__version__}
+            #
+            
+            import pyclesperanto_prototype as cle
+            import napari
+
+            if 'viewer' not in globals():
+                viewer = napari.Viewer()
+            """
+        ).strip()
+
+    @staticmethod
+    def imports():
+        return ""
+
+    @staticmethod
+    def subheader(step):
+        if "imread" in step.operation:
+            from textwrap import dedent
+
+            return dedent(
+                f"""
+                # todo: configure which inputs should be taken here.
+                # This takes the first selected layer's image data:
+                """).strip()
+        else:
+            return JythonGenerator.subheader(step)
+
+    @staticmethod
+    def operate(step) -> str:
+        from textwrap import dedent
+
+        if "imread" in step.operation:
+            return dedent(
+                f"""
+                {step.output} = list(viewer.layers.selection)[0].data
+                """).strip()
+        else:
+            return JythonGenerator.operate(step)
+
+    @staticmethod
+    def show(step):
+        if "imread" in step.operation:
+            return ""
+
+        code = ""
+        name = f"Result of {step.operation.replace('_', ' ')}"
+        if step.is_labels:
+            code = code + "viewer.add_labels("
+        else:
+            code = code + "viewer.add_image("
+
+        code = code + f"{step.output}"
+        code = code + f", name='{name}'"
+        if step.clims:
+            code = code + ", contrast_limits=" + str(step.clims)
+        code = code + ")"
+        return code
+
 
 @dataclass
 class Step:
@@ -95,6 +171,14 @@ class Pipeline:
             filename.write_text(code)
         return code
 
+    def to_napari_python(self, filename=None):
+        code = "\n".join(self._generate(NapariPythonGenerator)).replace("\n\n\n\n", "\n\n")
+        if filename:
+            filename = Path(filename).expanduser().resolve()
+            filename.write_text(code)
+        return code
+
+
     def to_notebook(self, filename=None, execute=False):
         import jupytext
 
@@ -132,8 +216,8 @@ class Pipeline:
         import dask
         steps = []
         for key in dask.order.order(graph):
-            op, inputs, args = graph[key]
-            steps.append(Step(operation=op.__name__, inputs=inputs, args=args, output=key))
+            op, inputs, args, is_labels = graph[key]
+            steps.append(Step(operation=op.__name__, inputs=inputs, args=args, output=key, is_labels=is_labels))
         return cls(steps=steps)
 
 
